C#######################################################################
C PSTSWM Version 1.0 (8/1/93)                                          #
C  A message-passing benchmark code and parallel algorithm testbed     #
C  that solves the nonlinear shallow water equations using the spectral#
C  transform method.                                                   #
C Written by:                                                          #
C  Patrick Worley of Oak Ridge National Laboratory                     #
C  Ian Foster of Argonne National Laboratory                           #
C Based on the sequential code STSWM 2.0 by James Hack and Ruediger    #
C  Jakob of the National Center for Atmospheric Research.              #
C Research and development funded by the Computer Hardware, Advanced   #
C  Mathematics, and Model Physics (CHAMMP) program of the U.S.         #
C  Department of Energy.                                               # 
C                                                                      #
C Questions and comments should be directed to worley@msr.epm.ornl.gov #
C Please notify and acknowledge the authors in any research or         #
C publications utilizing PSTSWM or any part of the code.               #
C                                                                      #
C NOTICE: Neither the institutions nor the authors make any            #
C representations about the suitability of this software for any       #
C purpose. This software is provided "as is", without express or       #
C implied warranty.                                                    #
C#######################################################################
      SUBROUTINE STEP(DTA, ALP, DALP, WTS, WTACSJ, ANNP1, A2NNP1, 
     &                TRIGS, MOUNT, TOPOSC, DIV, ZETA, PHI, UCOS, 
     &                VCOS, WS1, WS2, WS3, DIVSC, ZETASC, PHISC)
C
C This routine controls the timestepping process, and calls routine
C COMP1 to compute a single timestep. It updates the integers LNM1,
C LN and LNP1 as pointer offsets into a circular buffer of timelevels 
C for the prognostic variables divergence, vorticity and geopotential.
C Initialization is by two semi-implicit timesteps; normal time-
C stepping is by a centered leapfrog-scheme. 
C                                                                              
C called by: STSWM
C calls: COMP1
C
C---- Implicit None ----------------------------------------------------
C
      IMPLICIT NONE
C
C---- Model Parameters -------------------------------------------------
C
      INCLUDE 'params.i'
C
C---- Common Blocks ----------------------------------------------------
C
C problem resolution information
      INCLUDE 'problem.i'
C constants
      INCLUDE 'consts.i'
C domain decomposition information
      INCLUDE 'physical.i'
      INCLUDE 'spectral.i'
C time dependent fields
      INCLUDE 'tdvars.i'
C transform arrays
      INCLUDE 'trnsfm.i'
C
C---- Arguments --------------------------------------------------------
C
C     Input
C
C timestep
      REAL DTA
C associated Legendre polynomials
      REAL ALP(NFSPEC_S,NLLATH_S)
C deriv. associated Legendre polynomials
      REAL DALP(NFSPEC_S,NLLATH_S)
C Gaussian weights
      REAL WTS(NLAT)
C 1.0/(A*COS(LAT)**2)
      REAL WTACSJ(NLAT)
C A/(N*(N+1))
      REAL ANNP1(0:KK)
C (N*(N+1))/A**2
      REAL A2NNP1(0:KK)
C trigonometric function values used by RFTLON 
      COMPLEX TRIGS(NTRIGS)
C mountain height
      REAL MOUNT(NLLON_P,NLLAT_P)
C spectral coefficients of mountains
      COMPLEX TOPOSC(MXLSPEC_S)
C
C     Input/Output
C
C divergence field
      REAL DIV(MXLLON_P,NLVER_P,NLLAT_P,6)
C vorticity field
      REAL ZETA(MXLLON_P,NLVER_P,NLLAT_P,6)
C geopotential field
      REAL PHI(MXLLON_P,NLVER_P,NLLAT_P,6)
C eastward wind field (scaled by COS(THETA))
      REAL UCOS(MXLLON_P,NLVER_P,NLLAT_P)
C northward wind field (scaled by COS(THETA))
      REAL VCOS(MXLLON_P,NLVER_P,NLLAT_P)
C
C     Work Space
C
C work array 1
C (big enough for REAL (MXLLON_P,NLVER_P,NLLAT_P,8) 
C             and COMPLEX (MXLFC_S,NLVER_S,NLLAT_S,8)
C             and COMPLEX (4,MXLSPEC_S,NLVER_S))
      REAL WS1(1)
C work array 2
C (big enough for REAL (MXLLON_P,NLVER_P,NLLAT_P,8,BUFSWS2) 
C             and REAL (NLLON_P,NLVER_P,3) or
C             and COMPLEX (14,MXLFC_S,NLVER_S,NLLATH_S)
C             and REAL (0:KK,3))
      REAL WS2(1)
C work array 3
C (big enough for COMPLEX (MXLSPEC_S,NLVER_S,3,BUFSWS3))
      REAL WS3(1)
C
C     Output
C
C computed divergence at new timelevel
      COMPLEX DIVSC(MXLSPEC_S,NLVER_S)
C computed vorticity new timelevel
      COMPLEX ZETASC(MXLSPEC_S,NLVER_S)                        
C computed geopotential new timestep
      COMPLEX PHISC(MXLSPEC_S,NLVER_S)                        
C
C---- Local Variables --------------------------------------------------
C
C latitude, vertical layer, and longitude indices
      INTEGER J, JV, I
C temporary variable for holding AFC value during NSTEP = 1
      REAL TMPAFC
C
C---- Executable Statements --------------------------------------------
C
      IF (NSTEP .EQ. 1) THEN
C
C       Disable Asselin filter.
        TMPAFC = AFC
        AFC = 0.0
C
C       Set buffer pointers for first timesteps.
        LNM1   = 6
        LN     = 1
        LNP1   = LN
C
C       Copy prognostic variables into previous timelevel for smooth 
C       start.
        DO J=1,NLLAT_P
          DO JV=1,NLVER_P
            DO I=1,NLLON_P
              ZETA(I,JV,J,LNM1) = ZETA(I,JV,J,LN)
              DIV(I,JV,J,LNM1)  = DIV(I,JV,J,LN)
              PHI(I,JV,J,LNM1)  = PHI(I,JV,J,LN) 
            ENDDO
          ENDDO
        ENDDO
C
C       first (forward) timestep: DTA/2.0
C       (overwriting current timelevel and retaining initial data in 
C        old time level)
        CALL COMP1(DTA/2.0, ALP, DALP, WTS, WTACSJ, ANNP1, A2NNP1, 
     &             TRIGS, MOUNT, TOPOSC, DIV, ZETA, PHI, UCOS, 
     &             VCOS, WS1, WS2, WS3, DIVSC, ZETASC, PHISC)
C
C       second (centered) timestep: DTA
C       (again overwriting current timelevel and retaining initial data 
C        in old time level)
        CALL COMP1(DTA, ALP, DALP, WTS, WTACSJ, ANNP1, A2NNP1, 
     &             TRIGS, MOUNT, TOPOSC, DIV, ZETA, PHI, UCOS, 
     &             VCOS, WS1, WS2, WS3, DIVSC, ZETASC, PHISC)
C
C       Set buffer pointers for first leapfrog timestep (2.0*DTA)
C       so that update overwrites old time level.
C       LNM1   = 6
C       LN     = 1
        LNP1   = LNM1
C
C       Reenable Asselin filter.
        AFC = TMPAFC
c
      ELSE
C
C       subsequent (centered) timesteps: 2.0*DTA
        CALL COMP1(2.0*DTA, ALP, DALP, WTS, WTACSJ, ANNP1, A2NNP1, 
     &             TRIGS, MOUNT, TOPOSC, DIV, ZETA, PHI, UCOS, 
     &             VCOS, WS1, WS2, WS3, DIVSC, ZETASC, PHISC)
C
C       Advance circular buffer pointers.
        LNM1 = LN                                                           
        LN = LNP1
        LNP1 = LNM1
C
      ENDIF
C
      RETURN                                                                 
      END                                                                     
